@@ -17,10 +17,10 @@ from zope.annotation.interfaces import IAnnotations
 from zope.event import notify
 from zope.intid.interfaces import IIntIds
 from zope.schema import getFieldsInOrder
+from plone.dexterity.utils import createContentInContainer
 
 
 class ContentCopier(BaseContentCopier):
-
     def copyTo(self, container):
         context = aq_inner(self.context)
         wc = self._copyBaseline(container)
@@ -57,7 +57,7 @@ class ContentCopier(BaseContentCopier):
                 # Skip read-only fields
                 if field.readonly:
                     continue
-                if field.__name__ == 'id':
+                if field.__name__ == "id":
                     continue
                 try:
                     value = field.get(schema(self.context))
@@ -66,11 +66,11 @@ class ContentCopier(BaseContentCopier):
 
                 # TODO: We need a way to identify the DCFieldProperty
                 # fields and use the appropriate set_name/get_name
-                if name == 'effective':
+                if name == "effective":
                     baseline.effective_date = self.context.effective()
-                elif name == 'expires':
+                elif name == "expires":
                     baseline.expiration_date = self.context.expires()
-                elif name == 'subjects':
+                elif name == "subjects":
                     baseline.setSubject(self.context.Subject())
                 else:
                     field.set(baseline, value)
@@ -81,7 +81,13 @@ class ContentCopier(BaseContentCopier):
         wc_annotations = IAnnotations(self.context)
         baseline_annotations = IAnnotations(baseline)
 
-        baseline_annotations.clear()
+        # The next one is just wrong
+        # baseline_annotations.clear()
+
+        # Remove plone.folder.ordered.order from it to not mess with the original
+        if "plone.folder.ordered.order" in wc_annotations:
+            wc_annotations.pop("plone.folder.ordered.order")
+
         baseline_annotations.update(wc_annotations)
 
         # delete the working copy
@@ -93,13 +99,14 @@ class ContentCopier(BaseContentCopier):
         # reattach the source's workflow history, try avoid a dangling ref
         try:
             new_baseline.workflow_history = PersistentMapping(
-                baseline.workflow_history.items())
+                baseline.workflow_history.items()
+            )
         except AttributeError:
             # No workflow apparently.  Oh well.
             pass
 
         # reset wf state security directly
-        workflow_tool = getToolByName(self.context, 'portal_workflow')
+        workflow_tool = getToolByName(self.context, "portal_workflow")
         wfs = workflow_tool.getWorkflowsFor(self.context)
         for wf in wfs:
             if not isinstance(wf, DCWorkflowDefinition):
@@ -124,15 +131,14 @@ class ContentCopier(BaseContentCopier):
         id = intids.getId(context)
         # ask catalog
         catalog = component.getUtility(ICatalog)
-        relations = catalog.findRelations({'to_id': id})
-        relations = [i for i in relations
-                     if i.from_attribute == ITERATE_RELATION_NAME]
+        relations = catalog.findRelations({"to_id": id})
+        relations = [i for i in relations if i.from_attribute == ITERATE_RELATION_NAME]
         # do we have a baseline in our relations?
         if relations and not len(relations) == 1:
-            raise interfaces.CheckinException('Baseline count mismatch')
+            raise interfaces.CheckinException("Baseline count mismatch")
 
         if not relations or not relations[0]:
-            raise interfaces.CheckinException('Baseline has disappeared')
+            raise interfaces.CheckinException("Baseline has disappeared")
         return relations[0]
 
     def _getBaseline(self):
@@ -142,7 +148,7 @@ class ContentCopier(BaseContentCopier):
             baseline = intids.getObject(relation.from_id)
 
         if not baseline:
-            raise interfaces.CheckinException('Baseline has disappeared')
+            raise interfaces.CheckinException("Baseline has disappeared")
         return baseline
 
     def checkin(self, checkin_message):
@@ -152,15 +158,111 @@ class ContentCopier(BaseContentCopier):
         relation = self._get_relation_to_baseline()
         # publish the event for subscribers, early because contexts are about
         # to be manipulated
-        notify(event.CheckinEvent(self.context,
-                                  baseline,
-                                  relation,
-                                  checkin_message
-                                  ))
+        notify(event.CheckinEvent(self.context, baseline, relation, checkin_message))
         # merge the object back to the baseline with a copier
-        copier = component.queryAdapter(self.context,
-                                        interfaces.IObjectCopier)
+        copier = component.queryAdapter(self.context, interfaces.IObjectCopier)
         new_baseline = copier.merge()
         # don't need to unlock the lock disappears with old baseline deletion
         notify(AfterCheckinEvent(new_baseline, checkin_message))
         return new_baseline
+
+
+class FolderishContentCopier(ContentCopier):
+    def _copyBaseline(self, container):
+        obj = createContentInContainer(
+            container,
+            self.context.portal_type,
+            id="working_copy_of_{}".format(self.context.id),
+        )
+
+        # copy all field values from the baseline to the working copy
+        for schema in iterSchemata(self.context):
+            for name, field in getFieldsInOrder(schema):
+                # Skip read-only fields
+                if field.readonly:
+                    continue
+                if field.__name__ == "id":
+                    continue
+                try:
+                    value = field.get(schema(self.context))
+                except Exception:
+                    value = None
+
+                # TODO: We need a way to identify the DCFieldProperty
+                # fields and use the appropriate set_name/get_name
+                if name == "effective":
+                    obj.effective_date = self.context.effective()
+                elif name == "expires":
+                    obj.expiration_date = self.context.expires()
+                elif name == "subjects":
+                    obj.setSubject(self.context.Subject())
+                else:
+                    field.set(obj, value)
+
+        obj.reindexObject()
+
+        # copy annotations
+        wc_annotations = IAnnotations(obj)
+        baseline_annotations = IAnnotations(self.context)
+
+        wc_annotations.update(baseline_annotations)
+        # Remove plone.folder.order from it to not spoil the wc
+        if "plone.folder.ordered.order" in wc_annotations:
+            wc_annotations.pop("plone.folder.ordered.order")
+        if "plone.folder.ordered.pos" in wc_annotations:
+            wc_annotations.pop("plone.folder.ordered.pos")
+
+        return obj
+
+    def _replaceBaseline(self, baseline):
+        wc_id = self.context.getId()
+        wc_container = aq_parent(self.context)
+
+        # copy all field values from the working copy to the baseline
+        for schema in iterSchemata(baseline):
+            for name, field in getFieldsInOrder(schema):
+                # Skip read-only fields
+                if field.readonly:
+                    continue
+                if field.__name__ == "id":
+                    continue
+                try:
+                    value = field.get(schema(self.context))
+                except Exception:
+                    value = None
+
+                # TODO: We need a way to identify the DCFieldProperty
+                # fields and use the appropriate set_name/get_name
+                if name == "effective":
+                    baseline.effective_date = self.context.effective()
+                elif name == "expires":
+                    baseline.expiration_date = self.context.expires()
+                elif name == "subjects":
+                    baseline.setSubject(self.context.Subject())
+                else:
+                    field.set(baseline, value)
+
+        baseline.reindexObject()
+
+        # Move working children (newly created objects)
+        working_copy_children = [a for a in self.context.objectIds()]
+
+        clipboard = self.context.manage_cutObjects(working_copy_children)
+        baseline.manage_pasteObjects(clipboard)
+
+        # copy back annotations
+        wc_annotations = IAnnotations(self.context)
+        baseline_annotations = IAnnotations(baseline)
+
+        # Remove plone.folder.ordered.order from it to not mess with the original
+        if "plone.folder.ordered.order" in wc_annotations:
+            wc_annotations.pop("plone.folder.ordered.order")
+        if "plone.folder.ordered.pos" in wc_annotations:
+            wc_annotations.pop("plone.folder.ordered.pos")
+
+        baseline_annotations.update(wc_annotations)
+
+        # delete the working copy
+        wc_container._delObject(wc_id)
+
+        return baseline
