@@ -29,7 +29,7 @@ from plone.app.iterate import interfaces
 def handleBeforeCheckout(event):
     archiver = interfaces.IObjectArchiver(event.object)
     if archiver.isModified() or not archiver.isVersioned():
-        archiver.save('Baseline created')
+        archiver.save("Baseline created")
 
 
 def handleAfterCheckin(event):
